package facets.core.app;
import static facets.core.app.ActionAppSurface.CachingStyle.*;
import static facets.core.app.AppConstants.*;
import static facets.core.app.AppSurface.ContentStyle.*;
import static facets.util.app.AppValues.*;
import facets.core.app.Dialogs.Response;
import facets.core.superficial.Notice;
import facets.core.superficial.STarget;
import facets.util.Debug;
import facets.util.app.WatchableOperation;
import facets.util.app.AppServices;
import facets.util.app.AppValues;
import facets.util.app.AppWatcher;
import facets.util.app.AppWatcher.WatcherCoupler;
import facets.util.app.Events;
import facets.util.app.MemoryChecks;
import facets.util.app.ProvidingCache;
import facets.util.datatree.ValueNode;
import java.io.File;
import java.util.List;
/**
{@link AppSurface} defining application actions and management. 
<p>{@link ActionAppSurface} extends its superclass by 
<ul>
<li>composition with an {@link AppActions} that can launch dialogs and create root target 
 elements. 
<li>Interfacing with the {@link AppServices} passed to the constructor
</ul>
<p><b>Note:</b> {@link ActionAppSurface} is intended for use in concrete applications
 only via a facet builder subclass.    
 */
public abstract class ActionAppSurface extends AppSurface{
	/**
	{@link AppValues} that knows about {@link ActionAppSurface}. 
	 */
	public static class ActionAppValues extends AppValues{
		protected ActionAppSurface app;
		public ActionAppValues(Class appClass){
			super(appClass);
			
		}
		/**
		Attaches the {@link ActionAppValues} to its {@link ActionAppSurface}. 
		@param app was constructed from this {@link ActionAppValues} and calls this method
		immediately afterward. 
		 */
		protected void attachToApp(ActionAppSurface app){
			if(this.app!=null)throw new IllegalArgumentException(
					"Surface already attached in "+Debug.info(this));
			else this.app=app;
		}
	}
	/** Key for debug value. */
	final public static String 
		KEY_TIMEOUTS="timeoutChecks",
		KEY_TIMED_OUT="timedOut";
	private final static Object[]emptySources=new Object[]{DESKTOP};
	/** 
	Defines management style for {@link ProvidingCache}. 
	 */
	public enum CachingStyle{checkItemCount,checkItemSize,checkMemory}
	final public AppActions actions;
	public final AppWatcher watcher;
	public final MemoryChecks memory;
	/**
	Unique constructor. 
	@param spec set as final field
	@param services supplies an {@link AppWatcher} and application {@link ProvidingCache} 
	 */
	protected ActionAppSurface(AppSpecifier spec,final AppServices services){
		super(spec);
		if((this.actions=spec.newActions())==null)throw new IllegalArgumentException(
				"Null actions in "+Debug.info(this));
		else actions.attachToApp(this);
	  final ValueNode stateApp=values.state(PATH_CORE_APP),
	  	natureCore=values.nature(true);
		final boolean watching=natureCore.getBoolean(NATURE_RUN_WATCHED),
			debugWatch=watching&&natureCore.getBoolean(NATURE_RUN_WATCHED_DEBUG);
		((ActionAppValues)spec.values).attachToApp(this);
		if(debugWatch){
			if(!Debug.natureDebug)throw new IllegalStateException(
					"debugWatch=true in "+Debug.info(this));
			stateApp.put(KEY_TIMED_OUT,false);
			stateApp.put(KEY_TIMEOUTS,false);
		}
		watcher=!watching?null:services.coupleAppWatcher(new WatcherCoupler(){
			private WatchableOperation opThen;
			private void lastWarning(Exception e){
				services.warningCritical(title(),e,true);
			}
			@Override
			protected void handleException(List<WatchableOperation>ops,Exception e){
				if(false&&debugWatch)e.printStackTrace();
				else try{
					opThen=ops.get(0);
					actions.handleWatchCall(ops,
						e!=null?e:new Exception("Cause unknown"),
						false,!isBuilt());
				}catch(Exception ex){
					if(debugWatch)throw new RuntimeException(ex); 
					else lastWarning(e);
				}
			}
			@Override
			protected boolean retryCancel(List<WatchableOperation>ops){
				WatchableOperation op=ops.get(0);
				boolean firstRetry=opThen!=op;
				opThen=op;
				try{
					return actions.handleWatchCall(ops,null,firstRetry,
							!isBuilt());
				}catch(Exception e){
					if(debugWatch)throw new RuntimeException(e); 
					else lastWarning(e);
				}
				return false;
			}
			@Override
			protected void handleSystemTimeout(WatchableOperation watched,Thread timed,
					boolean wait){
				stateApp.put(KEY_TIMED_OUT,true);
				values.tryWrite("Timeout flag set. ");
				super.handleSystemTimeout(watched,timed,wait);
			}
			@Override
			protected boolean checkTimeouts(){
				return stateApp.getBoolean(KEY_TIMEOUTS);
			};
		});
		final CachingStyle caching=cachingStyle();
		memory=caching==checkItemCount?null:new MemoryChecks(values);
		services.setAppProvidingCache(caching==checkMemory?memory.newMemoryCache(watcher)
				:new ProvidingCache(100,watcher){
			protected boolean trace(){
				return true;
			}
			protected boolean checkItemBytes(){
				return caching!=checkItemCount;
			}
			protected long getBytesAvailable(long itemMems){
				int maxSafeMb=values.state(PATH_CORE_APP).getInt(MemoryChecks.NATURE_MEM_MB);
				return(calculateCacheMb(maxSafeMb)<<20)-itemMems;
			}
		});
		if(caching==checkMemory){
			memory.startChecks();
		  Events.traceEvent(">Checked memory"+memory.newStatusText(false));
		}
	}
	protected abstract CachingStyle cachingStyle();
	protected abstract long calculateCacheMb(int maxSafeMb);
	@Override
	public void notify(final Notice notice){
		if(false)actions.setWindowCloseLiveState();
		WatchableOperation op=new WatchableOperation("ActionAppSurface.notify"){
			@Override
			public void doSimpleOperation(){
				ActionAppSurface.super.notify(notice);
			}};
			if(true)runWatched(op);
			else op.doSimpleOperation();
	}
	/**
	Re-implementation calling {@link AppActions#surfaceRetargeted()}. 
	 */
	@Override
	final protected void surfaceRetargeted(){
		actions.surfaceRetargeted();
	}
	/**
	Implements abstract method. 
	<p>Returns targets created by {@link AppActions#newSurfaceRootElements()}. 
	 */
	@Override
	protected STarget[]lazySurfaceRootElements(){
		STarget[]targets=actions.newSurfaceRootElements();
		if(targets==null)throw new IllegalStateException(
				"Null targets in "+Debug.info(this));
		return targets;
	}
	/**
	Implements abstract method. 
	@return response of {@link AppActions#contentIsRemovable(String, AppSurface.Contenter)}
	 */
	@Override
	final public boolean contentIsRemovable(Contenter content){
		return actions.contentIsRemovable(AppActions.TITLE_CLOSE_CONTENT,
				content)==Response.Yes;
	}
	/**
	Implements interface method. 
	 */
	@Override
	final protected Object[]getOpeningContentSources(){
		if(contentStyle==DESKTOP)return emptySources;
		else if(!values.hasSystemAccess())return getFixedOpeningContentSources();
		File file=actions.getOpeningContentSourceFile();
		return file!=null&&file.isFile()?new Object[]{file}
			:getFixedOpeningContentSources();
	}
	protected Object[]getFixedOpeningContentSources(){
		return new Object[]{getInternalContentSource()};
	}
	/**
	Return a suitable source for default content, or the content itself.  
	<p>Called from {@link AppActions} and from default {@link #getFixedOpeningContentSources()}
	 */
	protected abstract Object getInternalContentSource();
	/**
	Supplies the {@link Dialogs} which will launch dialogs from this application surface. 
	 */
	public abstract Dialogs dialogs();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

