package facets.core.app;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STrigger;
import facets.core.superficial.TargetCore;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.SelectionView;
import facets.core.superficial.app.ViewableAction;
import facets.core.superficial.app.ViewableFrame;
import facets.core.superficial.app.ViewerTarget;
import facets.util.Debug;
import java.util.HashMap;
import java.util.Map;
/**
{@link ViewerTarget} that couples {@link ViewableAction}s to {@link STrigger}s. 
<p>{@link ActionViewerTarget} extends its superclass by defining
<ul><li>matching implementations of {@link ViewerTarget#newActionTriggers}
and {@link ViewerTarget#getTriggerAction}
<li>{@link #newActionTriggerGroup(String, ViewableAction[])} which can
create trigger groups for applications to return from {@link #newActionTriggerGroups(ViewableAction[])}
</ul> 
<p>The {@link Action} enumeration defines a range of {@link ViewableAction}s. 
<p>{@link ActionViewerTarget} is defined <code>abstract</code> to ensure that
the static class of application instances marks 
the dynamic type defined by the {@link ViewerTarget#actionTriggers} created
during construction.
 */
public abstract class ActionViewerTarget extends ViewerTarget{
	private final class ActionCoupler extends STrigger.Coupler{
		final ViewableAction action;
		ActionCoupler(ViewableAction action){
			this.action=action;
		}
		public void fired(STrigger t){
			ActionViewerTarget.this.actionTriggerFired(action);
		}
	}
	/**
	Defines a range of {@link ViewableAction}s. 
	 */
	public enum Action implements ViewableAction{
		COPY("C&opy"),CUT("Cu&t"),PASTE("P&aste"),PASTE_INTO("Paste &Child"),
		DELETE("De&lete"),EDIT("Mo&dify|Re&name"),
		ITERATE_FORWARD("Next|"+AppConstants.ARROW_RIGHT),
		ITERATE_BACK("Previous|"+AppConstants.ARROW_LEFT),
		SELECT_ALL("Select &All"),
		FOLDER_UP("Folder Up|Up"),FOLDER_OPEN("Folder Open|Open"),
		UNDO("Undo|Undo Edit"),REDO("Redo|Redo Edit");
		private final String title;
		Action (String title){this.title=title;}
		public String toString(){return title;}
	}
	private final Map<ViewableAction,STrigger>triggerMap=new HashMap();
	/**
	Unique constructor. 
	<p>All parameters are passed to the superclass. 
	 */
	public ActionViewerTarget(String title,ViewableFrame viewable,STarget views){
		super(title,viewable,views);
	}
	/**
	Implements abstract method. 
	<p>Returns triggers coupled via an internal subclass of 
	{@link facets.core.superficial.STrigger.Coupler}
	to <code>actions</code> and suitable for passing to {@link #getTriggerAction(STrigger)}. 
	 */
	final protected STrigger[]newActionTriggers(ViewableAction[]actions){
		STrigger[]triggers=new STrigger[actions.length];
		for(int i=0;i<triggers.length;i++)
			triggers[i]=new STrigger(actions[i].toString(),
					new ActionCoupler(actions[i]));
		return triggers;
	}
	/**
	Implements abstract method. 
	<p>Returns the {@link ViewableAction} coupled to <code>trigger</code> by 
	{@link #newActionTriggers(ViewableAction[])}  
	 */
	final protected ViewableAction getTriggerAction(STrigger trigger){
		return((ActionCoupler)trigger.coupler).action;
	}
	/**
	Creates a target group containing action triggers. 
	<p>Groups created are suitable for return by {@link #newActionTriggerGroups(ViewableAction[])}. 
	@param title for the group
	@param actions a subset of those passed to the constructor
	@return a group containing the members of {@link #actionTriggers}
	 coupled to <code>actions</code> in {@link #newActionTriggers(ViewableAction[])}
	 */
	protected final STarget newActionTriggerGroup(String title,ViewableAction[]actions){
		if(triggerMap.size()==0){
			for(int i=0;i<actionTriggers.length;i++)
				triggerMap.put(((ActionCoupler)actionTriggers[i].coupler).action,
						actionTriggers[i]);			
		}
		STarget[]triggers=new STarget[actions.length];
		for(int i=0;i<triggers.length;i++)
			if((triggers[i]=triggerMap.get(actions[i]))==null)
					throw new IllegalStateException("Null trigger in "+Debug.info(this));;
		return new TargetCore(title,triggers);
	}
	/**
	Convenience method for creating viewer arrangements. 
	<p>Creates {@link SAreaTarget}s each wrapping an instance of <i>the same</i> 
	trivial subclass of {@link ActionViewerTarget}; this method cannot therefore be 
	used where the {@link ViewableFrame} will define different {@link ViewableAction}s 
	for the underlying {@link SView}s.   
	@param viewable will be shared by all viewers created
	@param views each wrapping a {@link SView}
	 */
	public static SAreaTarget[]newViewerAreas(ViewableFrame viewable,
			SFrameTarget[]views){
		ViewerTarget[]viewers=new ViewerTarget[views.length];
		for(int i=0;i<viewers.length;i++){
			SFrameTarget frame=views[i];
			if(frame.framed instanceof SView)
				viewers[i]=new ActionViewerTarget(frame.title(),viewable,frame){};
			else throw new IllegalArgumentException("Bad framed in "+Debug.info(frame));
		}
		SAreaTarget[]areas=new SAreaTarget[viewers.length];
		for(int i=0;i<areas.length;i++)
			areas[i]=new SAreaTarget(views[i].title(),new ViewerTarget[]{viewers[i]});
		return areas;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

