package facets.core.app;
import facets.core.superficial.app.SSelection;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.OffsetPath;
import facets.util.Strings;
import facets.util.Util;
import facets.util.datatree.TypedNode;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
/**
Implements {@link SSelection} using {@link OffsetPath}s. 
<p>The immutable {@link #paths} members of {@link PathSelection} should describe 
paths rooted on {@link #content()}. 
	 */
public final class PathSelection implements SSelection{
	/** 
	The {@link OffsetPath}s passed to the constructor and rooted on {@link #content()}. 
	 */
	final public OffsetPath[]paths;
	final private transient Object content;
	/**
	Core constructor. 
	@param paths set as {@link #paths}; they be rooted on <code>content</code>, 
	which is returned as {@link #content()}
	 */
	public PathSelection(Object content,OffsetPath...paths){
		this.content=content;
		this.paths=paths;
		if(paths.length==0)throw new IllegalArgumentException(
				"Empty paths in "+Debug.info(this));
	}
	/**
	Creates a selection with a single, single-membered path. 
	@param content is passed to {@link #newSingle(Object, OffsetPath)}
	 together with {@link OffsetPath#singleMembered}
	 */
	public static PathSelection newMinimal(Object content){
		return newSingle(content,OffsetPath.singleMembered);
	}
	/**
	Creates a selection with a single path. 
	<p>Content is passed to {@link #PathSelection(Object,OffsetPath[])}
	 together with an {@link OffsetPath}[] containing <code>path</code>
	 */
	public static PathSelection newSingle(Object content,OffsetPath path){
		return new PathSelection(content,new OffsetPath[]{path});
	}
	public static PathSelection procrust(PathSelection src,Object to){
		List<OffsetPath>out=new ArrayList();
		for(OffsetPath in:src.paths)out.add(in.procrusted(src.content,to));
		return new PathSelection(to,out.toArray(new OffsetPath[]{}));
	}
	public static Object[]pathMembers(PathSelection selection,int pathAt){
		return selection.paths[pathAt].members(selection.content());
	}
	/**
	Implements interface method. 
	@return the {@link OffsetPath#target(Object)}s of {@link #paths}
	 */
	final public Object[]selections(){
		Object[]selected=new Object[paths.length];
		for(int i=0;i<selected.length;i++)selected[i]=paths[i]==OffsetPath.empty?
			OffsetPath.singleMembered:paths[i].target(content);
		return selected;
	}
	/**
	Implements interface method. 
	@return the first member of {@link #selections()}
	 */
	final public Object selection(){
		return selections()[0];
	}
	final public Object content(){
		if(content==null)throw new IllegalStateException("Null content (deserialized) in "+Debug.info(this));
		return content;
	}
	public String toString(){
		return Debug.info(this)+"\ncontent="+Debug.info(content)+" selected="+
			Objects.toLines(selections())+
			"\npaths[0]=" +Debug.info(paths[0])
					+" [" +Strings.intsString(paths[0].offsets)+"]"
			+(true?"":Objects.toLines(paths[0].members(content)));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
