package facets.core.app;
import static facets.core.app.ActionViewerTarget.Action.*;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.ViewableAction;
import facets.core.superficial.app.ViewableFrame;
import facets.util.Debug;
import facets.util.Stateful;
/**
{@link ViewableFrame} with {@link Stateful} content. 
<p>{@link StatefulViewable} extends its superclass by 
requiring both its {@link #framed} content and all members of its {@link #selection()} 
to be {@link Stateful}, enabling the provision of 
<ul>
<li>data transfer using a {@link Clipper} connection to the system clipboard
and drag and drop implementation 
<li>state management (undo/redo)
</ul> 
 */
public abstract class StatefulViewable<S extends Stateful> extends StatefulViewableCore{
	/**
	Can supply a {@link StatefulViewable} with a suitable {@link Clipper}. 
	 */
	public interface ClipperSource{
		Clipper newClipper(StatefulViewable viewable);
	}
	/**
	Can copy and paste the content of a {@link StatefulViewable}. 
	 */
	public interface Clipper{
		/**
		Place the currently selected content on the clipboard. 
		 */
		void copySelection();
		/**
		 Are the current clipboard contents suitable for pasting? 
		 */
		boolean canPaste();
		/**
		 Attempt to create content from the current clipboard contents. 
		 @return the content, or <code>null</code> on failure
		 */
		Stateful[]newStatefuls();
	}
	protected final Clipper clipper;
	/**
	Unique constructor. 
	@param title passed to superclass
	@param content passed to superclass
	@param clipperSource if non-<code>null</code> will be used for data transfer
	 */
	protected StatefulViewable(String title,S content,ClipperSource clipperSource){
		super(title,content);
		clipper=clipperSource==null?null:clipperSource.newClipper(this);
	}
	public final void copyStatefulSelection(){
		if(clipper==null)throw new IllegalStateException("Null clipper in "+Debug.info(this));
		clipper.copySelection();
	}
	final protected boolean canPaste(){
		return clipper==null?false:clipper.canPaste();
	}
	final protected Stateful[]newPasteStatefuls(){
		if(clipper==null)throw new IllegalStateException("Null clipper in "+Debug.info(this));
		return clipper.newStatefuls();
	}
	/**
	Create a textual representation of one or more {@link Stateful}s for use in 
	data transfer.
	<p>May be called (indirectly) from {@link #copyStatefulSelection()}; 
	default returns a debug string. 
	@param selected are the currently selected content 
	 */
	public String newStatefulsText(S[]selected){
		return Debug.info(selected);
	}
	/**
	Does the text passed seem to represent suitable content? 
	<p>Used to determine the live state of the viewer 'Paste' target; default
	return <code>false</code>.
	@param text is currently on the system clipboard
	 */
	public boolean textSeemsPastable(String text){return false;}
	/**
	Attempt to create content from a text representation. 
	<p>Called by {@link #newPasteStatefuls()}; 
	may return <code>null</code> (the default) to signal failure. 
	@param text is currently on the system clipboard
	 */
	protected S[]newPasteTextStatefuls(String text){
		return null;
	}
	public void iterateSelection(boolean forward){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public void insertStatefuls(Stateful...stateful) {
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public void deleteSelection(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public boolean editSelection(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	protected void copyContentState(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	/**
	Re-implementation that understands {@link ActionViewerTarget.Action}s. 
	 */
	public boolean actionIsLive(SViewer viewer,ViewableAction action){
		return action==UNDO?states.canUndo() 
			:action==REDO?states.canRedo() 
			:super.actionIsLive(viewer,action);
	}
	/**
	Re-implementation that understands {@link ActionViewerTarget.Action}s. 
	 */
	protected void actionTriggered(SViewer viewer,ViewableAction action){
		if(action==CUT||action==COPY)copyStatefulSelection();
		boolean edited=action==PASTE||action==CUT||action==DELETE||action==UNDO||action==REDO;
		if(action==SELECT_ALL)defineSelection(SELECT_ALL);
		else if(action==ITERATE_FORWARD||action==ITERATE_BACK)
			iterateSelection(action==ITERATE_FORWARD);
		else if(action==PASTE)insertStatefuls(newPasteStatefuls());
		else if(action==CUT||action==DELETE)deleteSelection();
		else if(action==EDIT)edited|=editSelection();
		else if(action==UNDO)states.undo();
		else if(action==REDO)states.redo();
		if(edited)updateAfterEditAction();
	}
	final public void updateAfterEditAction(){
		copyContentState();
		((S)framed).updateStateStamp();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

