package facets.core.app;
import static facets.util.Debug.*;
import facets.core.app.AppSurface.ContentRoot;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SContenter;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.ViewableAction;
import facets.core.superficial.app.ViewableFrame;
import facets.core.superficial.app.ViewerTarget;
import facets.util.Debug;
import facets.util.FileSpecifier;
import facets.util.Tracer;
import facets.util.app.Events;
import java.io.IOException;
/**
{@link SContenter} that builds content roots containing viewers. 
<p>{@link ViewerContenter} extends its superclass to build 
  one or more roots containing viewers exposing content created from a source
  passed to its constructor. 
<p>Its implementation of {@link #newContentRoot(boolean, boolean)}
defines template methods encapsulating two distinct approaches to the
build process.
<p>Since {@link ViewerContenter} primarily creates content for 
{@link AppSurface}, it provides a basic implementation of 
 {@link AppSurface.Contenter}; it also defines methods for saving content back
 to a source such as a file than can also function as a sink.
*/
public abstract class ViewerContenter extends Tracer 
		implements SContenter, AppSurface.Contenter{
 	public static final ViewableFrame VIEWABLE_NONE=new ViewableFrame("No viewable set",""){};
	/**
		Can create content. 
	<p>{@link ContentSource}s can encapsulate generation of 
	content for use by implementations of {@link SContenter#newContentRoot(boolean, boolean)}. 
	 */
	public static interface ContentSource{
		/**
		Create new content. 
		 */
		Object newContent();
	}
	private Object sourceOrSink;
	private ViewableFrame viewable=VIEWABLE_NONE;
	/**
	Unique constructor. 
	@param sourceOrSink passed by {@link #newContentRoot(boolean, boolean)} to 
	{@link #newContentViewable(Object)}
	 */
	protected ViewerContenter(Object sourceOrSink){
		if((this.sourceOrSink=sourceOrSink)==null)
			throw new IllegalArgumentException("Null source or sink in "+info(this));
	}
	/**
	Implements abstract method. 
	<p>Calls template methods as follows:
	<ol>
	<li>{@link #newViewableContentRoot(Object, boolean)} which can be implemented to 
	create a content root complete with attached facet; such a root
	is then returned without the remaining methods being called.  
  <li>{@link #newContentViewable(Object)} to create content from the source
  passed to the constructor and wrap it in a {@link ViewableFrame}. 
  <li>{@link #newContentViewers(ViewableFrame)}to define one 
    or more {@link ViewerTarget}s for incorporation in a 
    content area root, with multiple viewers contained in {@link SAreaTarget}s 
    so as to meet the contract of {@link facets.core.superficial.app.SAreaTarget}.
	<li>{@link #newContentRootTitle(ViewableFrame)} and 
	{@link #lazyContentRootElements(SAreaTarget)} to specify the remaining
	features of an {@link SAreaTarget} to be returned as the root of 
	the viewers area tree, followed by 
	{@link #newAppContentRoot(String, FacetedTarget[])} to create the root
	<li>(if <code>faceted</code> is <code>true</code>)
	{@link #attachContentRootFacets(SAreaTarget)} 
	to create and attach facet to the area tree.  
	</ol>
	@param faceted if <code>false</code> can signal a fresh invocation that requires
	return of an unfaceted root wrapping the existing viewable
	 */
	public final SAreaTarget newContentRoot(boolean faceted, boolean shareSelection){
	  Events.traceEvent(">Creating content root in "+info(this));
	  SAreaTarget root=newViewableContentRoot(sourceOrSink,faceted);
		if(root!=null){
			viewable=((ViewerTarget)root.activeFacet()).viewable;
		  Events.traceEvent(">Created content root " +info(root));
			return root;
		}
	  if(viewable==VIEWABLE_NONE){
		  Events.traceEvent(">Creating viewable from "+info(sourceOrSink));
	  	viewable=newContentViewable(sourceOrSink);
	  }
	  if(viewable==VIEWABLE_NONE)throw new IllegalStateException("No viewable in "+Debug.info(this));
	  else Events.traceEvent(">Creating viewers for "+info(viewable));
	  FacetedTarget[]viewers=newContentViewers(viewable);
	  if(viewers==null)throw new IllegalStateException("Invalid null viewers in "+info(this));
	  else if(!((viewers.length==1&&viewers[0]instanceof ViewerTarget)||
	  		viewers instanceof SAreaTarget[]))
	  	throw new IllegalStateException("Invalid viewer array in "+info(this));
	  String rootTitle=newContentRootTitle(viewable);
	  if(rootTitle==null||rootTitle.equals(""))
	  	throw new IllegalStateException("Null or empty content root title in "+info(this));
		root=newAppContentRoot(rootTitle,viewers);
	  if(root==null)throw new IllegalStateException("Invalid null root in "+info(this));
	  else Events.traceEvent(">Created content root " +info(root));
		if(!faceted)return root;
		attachContentRootFacets(root);
	  try{
	  	root.attachedFacet();
	  }
	  catch(Exception e){	  	
	  	throw new IllegalStateException(Debug.info(root)+" has no facet in "+info(this));
	  }
	  Events.traceEvent(">Attached root facet to " +info(root));
		return root;
	}
	/**
	May create an area target tree complete with facets. 
	<p>Called by {@link #newContentRoot(boolean, boolean)} with the object passed to the 
	constructor; should return an {@link SAreaTarget} to meet the same contract. 
	<p>Default returns <code>null</code>, requiring valid implementations of 
	subsequent methods.  
	@param sourceOrContent may be either a content source such as a file, or
	the content itself. 
	 @param faceted if <code>true</code>, the root returned 
	 should have a facet attached
	 */
	protected SAreaTarget newViewableContentRoot(Object sourceOrContent, 
			boolean faceted){
		return null;
	}
	/**
	Create a {@link ViewableFrame} framing content. 
	<p>Called by the first invocation of {@link #newContentRoot(boolean, boolean)}, 
	with the content or source passed to the constructor. 
	<p>A valid implementation of this method should create default content and/or 
	content from a file or database connection, possibly using an implementation of 
	{@link ViewerContenter.ContentSource} passed to the constructor.
	<p>The {@link ViewableFrame} returned will be passed to 
	{@link #newContentViewers(ViewableFrame)}. 
	<p>Default implementation is an invalid stub. 
	@param source must be capable either of defining or of creating content; it 
	should not generally be the content itself 
	 */
	protected ViewableFrame newContentViewable(Object source){
		throw new RuntimeException("Not implemented in "+this);
	}
	/**
	Create a {@link FacetedTarget}[] defining an arrangement of 
	 viewers for <code>viewable</code>. 
	<p>Called by {@link #newContentRoot(boolean, boolean)} with the {@link ViewableFrame} 
	returned by {@link #newContentViewable(Object)}. 
	 <p>The {@link FacetedTarget}s returned must comprise either 
	 <ul><li>a single {@link facets.core.superficial.app.ViewerTarget}, or
	 <li> any number of {@link SAreaTarget}s defining an arrangement of 
	 {@link facets.core.superficial.app.ViewerTarget}s. 
	</ul>
	<p>Default implementation is an invalid stub;
	 {@link ActionViewerTarget#newViewerAreas(ViewableFrame, SFrameTarget[])} 
	 can be used for implementations 
	 where viewers share the same {@link ViewableAction}[].
	 @param viewable was returned by {@link #newContentViewable(Object)}
	 and should be returned as the <code>viewable</code> of all 
	 {@link facets.core.superficial.app.ViewerTarget}s 
	 constructed in this method.    
	 */
	protected FacetedTarget[]newContentViewers(ViewableFrame viewable){
		throw new RuntimeException("Not implemented in "+this);
	}
	/**
	Attach viewer and area facets to the tree headed
	by <code>rootArea</code> using a suitable facet builder. 
	<p>Called by the first invocation of {@link #newContentRoot(boolean, boolean)}, 
	with the {@link SAreaTarget} returned by 
	{@link #newAppContentRoot(String, FacetedTarget[])} 	
	<p>Default implementation is an invalid stub. 
	@param rootArea was returned by {@link #newContentRoot(boolean, boolean)}
	 */
	protected void attachContentRootFacets(SAreaTarget rootArea){
		throw new RuntimeException("Not implemented in "+this);
	}
	/**
	Return the title for a content root. 
	<p>The default implementation returns the <code>title</code> of <code>viewable</code>.
	@param viewable was constructed in {@link #newContentViewable(Object)}
	 */
	protected String newContentRootTitle(ViewableFrame viewable){
		return viewable.title();
	}
	/**
	Creates a content root suitable for multi-content application. 
	<p>Creates a subclass of {@link AppSurface.ContentRoot} 
	returning <code>elements</code> defined in 
	{@link #lazyContentRootElements(SAreaTarget)}, with <code>viewers</code>
	as its children. 
	<p>Called by {@link #newContentRoot(boolean, boolean)}, should also be called from 
	implementations of {@link #newViewableContentRoot(Object, boolean)}
	 @param title was returned by {@link #newContentRootTitle(ViewableFrame)}
	 @param viewers were created in {@link #newContentViewers(ViewableFrame)}
	 */
	final protected ContentRoot newAppContentRoot(String title,
			FacetedTarget[]viewers){
		return new AppSurface.ContentRoot(title,viewers,this){
			protected STarget[]lazyElements(){
				return lazyContentRootElements(this);
			}
		};
	}
	/**
	Return elements for the content root. 
	<p>The default implementation returns an empty <code>Target[]</code> to
	meet the contract of <code>TargetCore.lazyElements</code>. 
	@param root contains the viewers returned by 
		{@link #newContentViewers(ViewableFrame)}.
	 */
	protected STarget[]lazyContentRootElements(SAreaTarget root){
		return new STarget[]{};
	}
	public void alignAddedRoot(SAreaTarget existing,SAreaTarget added){}
	/**
	Implements interface method. 
	<p>Empty implementation.  
	 */
	public void rootRetargeted(SContentRootTargeter t){}
	/**
	Returns the {@link ViewableFrame} framing content generated by the 
	<code>source</code> of this {@link ViewerContenter}.
	<p>The {@link ViewableFrame} is unique to this instance.  
	 */
	final public SFrameTarget contentFrame(){
		if(viewable==null)throw new IllegalStateException("No viewable in "+Debug.info(this));
		return viewable;
	}
	/**
	Implements interface method. 
	<p>Default returns <code>true</code>. 
	 */
	public boolean hasChanged(){return true;}
	public void wasAdded(){}
	public void wasRemoved(){}
	/**
	 Implements abstract method. 
	 <p>Returns the title of {@link #contentFrame()}. 
	 */
	public String title(){
		return contentFrame().title();
	}
  /**
	Data sink to which the {@link ViewerContenter} can be persisted. 
	<p>Set during construction to the source.
	 */
	final public Object sink(){
		return sourceOrSink;
	}
	/**
	Sets the data sink. 
	@param sink becomes the new {@link #sink()}
	@return <code>true</code> by default; return <code>false</code> to signal 
	unacceptable sink
	 */
	public boolean setSink(Object sink){
		if(sink==null)throw new IllegalArgumentException("Null sink in "+info(this));
		this.sourceOrSink=sink;
		return true;
	}
	/**
	Attempt to save to a sink. 
	<p>Default is invalid stub.
	 @param sink typically a file
	 @throws IOException
	 */
	public void saveToSink(Object sink)throws IOException{
		if(true)throw new IOException("No save in "+info(this));		
		else throw new RuntimeException("Not implemented in "+info(this));		
	}
	public FileSpecifier[]sinkFileSpecifiers(){
		throw new RuntimeException("Not implemented in "+this);
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
