package facets.core.app.avatar;
import facets.util.geom.Line;
import facets.util.geom.Point;
import facets.util.shade.Shade;
/**
Factory for creating and transforming avatar painters. 
<p>A {@link PainterSource} defines methods for use by 
{@link facets.core.app.avatar.AvatarPolicy} and {@link facets.core.app.avatar.DragPolicy} that
<ul>
<li>create {@link Painter} instances 
<li>create and apply {@link Transform} instances
</ul> 
<p>Instances of {@link PainterSource} can be supplied by the facet builder via
viewer facets. 
 */
public interface PainterSource{
  /**
  Marks an immutable transform.   
  <p>{@link Transform} provides type safety
  while manipulating transforms created by the <i>transformX</i> methods 
  of {@link PainterSource} before they are passed to 
  {@link PainterSource#applyTransforms(facets.core.app.avatar.PainterSource.Transform[],boolean, Painter...)}. 
   */
  public interface Transform{}
	int NO_PATTERN=0,HAIRLINE=-1;
	Painter textTooltip(String text,double x,double y);
	Painter textLineAt(String text,String face,int points,Shade shade,double x,
			double y);
	Painter textOutline(String text,String face,int points,boolean bold,
			boolean italic,Shade fill,Shade pen);
	double textLength(String text,String face,int size,boolean bold,boolean italic);
	Painter rectangle(double x,double y,double width,double height,Shade pen);
	Painter bar(double x,double y,double length,double width,Shade fill,
			boolean pickable);
	Painter stretchMark(double height,double length,double width,Shade fill);
	Painter turnMark(double size,double width,Shade fill);
	/**
	Create a (maybe pickable) painter to mark a position. 
	<p>The appearance of the painter is defined by the implementation. 
	@param at the position
	@param fill for the painting
	@param pickable should the mark be pickable?
	 */
	Painter pointMark(Point at,Shade fill,boolean pickable);
	/**
	Create a possibly-pickable line painter. 
	@param line to be painted
	@param shade for the painting
	@param style defines the pattern or stroke
	@param pickable defines line behaviour
	 */
	Painter line(Line line,Shade shade,int style,boolean pickable);
	Painter mastered(PainterMaster master);
	/**
	Create non-pickable painter of the lines passed. 
	@param lines will be painted at minimum width 
	@param shade for the painting
	 */
	Painter backgroundLines(Line[]lines,Shade shade);
	Painter bundle(Painter[]painters,Object...values);
	Transform transformTurn(double radians,double atX,double atY);
	Transform transformScale(double xy);
	Transform transformAt(double x,double y);
	/**
	Apply transforms to painters. 
	@param transforms should be applied to each member of <code>painters</code>
	@param painters to be transformed
	 */
	void applyTransforms(Transform[]transforms,boolean concatenate,Painter...painters);
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
