package facets.core.superficial;
import facets.core.superficial.Notifying.Impact;
import facets.util.Debug;
import facets.util.Identified;
import facets.util.Util;
import facets.util.datatree.TypedNode;

import java.io.Serializable;
/**
Potential element of the Facets notification tree. 
<p>{@link Notifying} is the root of the Facets type hierarchy 
  for both targets and targeters. 
<p>Whenever a target acts on the element it represents in the 
  application (usually though not necessarily in response to widget 
  input relayed by a surface facet) the application root must 
  be notified that a retargeting is needed to ensure view and 
  control consistency. 
<p>Notification is triggered in the Facets implementation of by application code calling <code>notifyMonitor</code> on the 
  target, and relayed to the application root via a tree of {@link Notifying}s, 
  all either {@link facets.core.superficial.STarget}s or {@link facets.core.superficial.STargeter}s. 
<p>A {@link Notifying} is attached to the tree using <code>setMonitor</code> 
  to set one of the following as its notification monitor: 
<ul>
  <li>for a {@link STarget} for which {@link TargetCore#notifiesTargeter()} 
  returns <code>false</code>,
    the {@link STarget} of which it is a child</li>
  <li>for a {@link STarget} for which {@link TargetCore#notifiesTargeter()} 
  returns <code>true</code>, the {@link STargeter} 
    of which it is the target</li>
  <li>for a {@link STargeter}, its parent in the targeter 
    tree; for the root of the targeter tree, the surface itself</li>
</ul>
<p>In all three cases the monitor is set during retargeting; 
applications need only ensure that notification 
  is triggered whenever a target acts on the application element 
  it represents. 

 */
public interface Notifying extends Notifiable{
	/**
  Allows a {@link Notifying} to refine its notification.
  <p>The members of {@link Impact} can define the scope of the event 
  triggering a notifcation;   
  each member guarantees that its impact will be less than any member
  with higher {@link #ordinal()} 
<ul>
	<li>{@link #FLASH} no state change; typically a status message  
	<li>{@link #ACTIVE} active state eg of viewers may have changed
	<li>{@link #SELECTION} content selection state may have changed
	<li>{@link #CONTENT} content state may have changed
	<li>{@link #DEFAULT} any application element or state may have changed
	</ul> 
	 */
  public enum Impact{
  	FLASH,
  	ACTIVE,
  	SELECTION,
  	CONTENT,
  	DEFAULT,
	}
	/**
	Attach the {@link Notifying} to a {@link Notifiable} parent 
	in the notification tree. 
	 
			 */
  void setMonitor(Notifiable monitor);
	/**
	Call <code>notify</code> on the parent monitor in the notification tree, 
	typically with itself as parameter to the {@link Notice} passed. 
	<p>Has no effect if no monitor has been set.
	<p>This is the usual means to trigger a surface retargeting, being called
	by an exposing {@link SFacet} on its target. 
	@param impact suggests the level of retargeting required 
	 
  		 */
  void notifyMonitor(Impact impact);
	/**
	As last set by <code>setMonitor</code>. 
	<p>Complains if none set. 
	 */
	Notifiable monitor();
	/**
	Allows this {@link Notifying} to specify a default {@link Impact}.
	@return typically {@link Impact#DEFAULT} 
	 */
	Impact impact();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

