package facets.core.superficial;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Objects;
import facets.util.Times;
import facets.util.Titled;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.datatree.TypedNode;
/**
{@link STarget} representing one or more indices into a list of items. 
<p>{@link SIndexing} represents a list of items to be exposed 
  to user view and control in the surface, together with an index/indices 
  into that list; application-specific mechanism and policy can be defined in 
	a {@link facets.core.superficial.SIndexing.Coupler}. 
 */
final public class SIndexing extends TargetCore{
	/**Allows an indexing to be functionally empty. */
	public static final Object[]NO_INDEXABLES={"Not indexable"};
	private static final String NOT_INDEXED="Not indexed";
	/**
	Connects an {@link SIndexing} to the application. 
	<p>A {@link Coupler} supplies client-specific mechanism and 
	  policy for an {@link SIndexing}. 
	 */
	public static class Coupler extends Tracer implements TargetCoupler{
		/**
		Returns strings to represent the indexables.  
		<p>These will appear in facet lists, or as labels for menu items or 
		radio buttons. 
		<p>The default returns titles based on the type of the {@link SIndexing}'s 
		<code>indexables</code>:
		<ul>
		<li>if they are {@link String}s, returns them</li> 
		<li>if they are {@link facets.util.Titled}s, returns their <code>title</code>  properties</li> 
		<li>otherwise, returns their <code>toString</code> properties</li> 
		<li>if they the {@link SIndexing#NO_INDEXABLES} constant, returns an 
		empty {@link String}[].
		</ul> 		 
		 */
		public String[]newIndexableTitles(SIndexing i){
		  Object[]indexables=i.indexables();
			if(indexables==NO_INDEXABLES)return new String[]{};
			else if(indexables instanceof String[])return(String[])indexables;
		  String[]titles=new String[indexables.length];
		  boolean titled=indexables instanceof Titled[];
		  for(int t=0;t<titles.length;t++)
		    titles[t]=titled?((Titled)indexables[t]).title():
		      indexables[t].toString();
		  return titles;
		}
		/**
		Called whenever an index changes. 
		 */
		public void indexSet(SIndexing i){}
		/**
		Can return dynamic indexables. 
		<p>Called by {@link SIndexing#indexables()} if none set during construction. 
		@return a non-empty array
		 */
		public Object[]getIndexables(){
			throw new RuntimeException("Not implemented in "+Debug.info(this));
		}
		/**
		Returns titles for iterating buttons or menu items.  
    <p>Default implementation returns "Previous","Next". 
    */
		public String[]iterationTitles(SIndexing i) {
			return new String[]{"Previous|\u25c4","Next|\u25ba"};
		}
		/**
		Can the indexing loop round when iterating through its indexables?
		 <p>Default returns <code>false</code>.
		 */
		public boolean canCycle(SIndexing i){return false;}
		/**
    Returns an array defining the 'enabled' state of each item. 
    <p>May be needed for menu items or radio buttons. 
    Default implementation returns states to match <code>i</code>.
     */
    public boolean[]liveStates(SIndexing i){
      boolean lives[]=new boolean[newIndexableTitles(i).length],
      	live=i.isLive();
      for(int e=0;e<lives.length;e++)lives[e]=live;
      return lives;
    }
	}
	/**
	The <code>Indexing.Coupler</code> passed to the core constructor. 
	 */
	public final Coupler coupler;	
	private int[]indices=null;
	private transient Object[]indexables,indexings;  
	/**
	Core constructor. 
	<p>Declared visibly for documentation purposes only. 
	 @param title passed to superclass
	 @param indexables the objects to be indexed; may not be empty but 
	 may be <code>null</code> in which case they are 
	 read dynamically from the coupler
	 @param coupler supplies application-specific mechanism and policy
	 */
	protected SIndexing(String title,Object[]indexables,Coupler coupler){
		super(title);this.coupler=coupler;
		if(indexables!=null&&indexables.length==0)
			throw new IllegalArgumentException("Null or empty indexables in "+Debug.info(this));
		this.indexables=indexables;
	}
	/**
	Convenience constructor setting initial index. 
	 @param index the initial index
	 */
	public SIndexing(String title,Object[]indexables,int index,Coupler coupler){
		this(title,indexables,coupler);
		setIndex(index);
	}
	/**
	Convenience constructor setting initial indexed. 
	 @param toIndex the object to have the initial index
	 */
	public SIndexing(String title,Object[]indexables,Object toIndex,
			Coupler coupler){
		this(title,indexables,coupler);
		setIndexed(toIndex);
	}
	/**
	Convenience constructor forcing dynamic read of indexables. 
	 */
	public SIndexing(String title,Coupler coupler){
		this(title,null,coupler);
		setIndex(0);
	}
  /**
	The first index into the <code>indexables</code>. 
		 */
	public int index(){
		if(indices==null||indices.length==0)throw new IllegalStateException(
				"Null or empty indices in "+Debug.info(this));
		return indices[0];
	}
  /**
	The indices last set into the <code>indexables</code>. 
		 */
	public int[]indices(){
		if(indices==null)throw new IllegalStateException(
				"Null indices in "+Debug.info(this));
		return indices;
	}
	/**
	Sets the index to that of the item passed.
	<p>Complains if the item is not a member of <code>indexables</code>.  
		 */
  public void setIndexed(Object toSet){
    Object[]indexables=indexables();
    for(int i=0;i<indexables.length;i++)
      if(indexables[i].equals(toSet))setIndex(i);
    if(!indexed().equals(toSet))
      throw new IllegalStateException(Debug.info(toSet)+"\n\t in "+Debug.info(this)
          +" not indexed in: \n"+Debug.arrayInfo(indexables));
  }
  /**
	The items exposed to indexing. 
	<p>These will either have been set during construction or be read 
	dynamically from the coupler. 
		 */
	public Object[]indexables(){
		Object[]indexables=this.indexables!=null?this.indexables
				:coupler.getIndexables();
	  if(indexables==null)throw new IllegalStateException(
	  		"Null indexables in "+this);
	  return indexables.length==0?NO_INDEXABLES:indexables;
	}
	/**
	Sets a single index into the <code>indexables</code>. 
		 */
	public void setIndex(int index){
		setIndices(new int[]{index});
	}
	/**
	Sets indices into the <code>indexables</code>. 
	@param indices may be empty but may not be <code>null</code>
		 */
	public void setIndices(int[]indices){
		if(indices==null)throw new IllegalArgumentException(
				"Null indices in "+Debug.info(this));
	  boolean first=this.indices==null;
	  for(int i=0;indices.length>0&&i<indices.length;i++)
	  	if(indices[i]<0)throw new IllegalArgumentException("Bad index in "+Debug.info(this));
	  this.indices=indices;
	  if(indexings!=null){
	    Object[]indexables=indexables(),old=indexings;
	    indexings=new Object[old.length];indexings[0]=indexables[indices[0]];
	    for(int i=0,indexion=1;i<old.length;i++)
	      if(old[i]!=indexings[0])indexings[indexion++]=old[i];
	  }
	  if(!first)coupler.indexSet(this);
	}
	/**
	The item denoted by the current <code>index</code>. 
		 */
	public Object indexed(){
	  if(indices==null)throw new IllegalStateException("No index in "+Debug.info(this));
	  return indices.length==0?NOT_INDEXED:indexables()[index()];
	}
	/**
	The <code>indexables</code> ordered by most recently indexed. 
		 */
	public Object[]indexings(){
	  if(indexings==null)throw new IllegalStateException("Null indexings in "+this);
	  return indexings;
	}
	public String toString(){
		return super.toString()+" "+indices().length;
	}
  protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    if(indexables!=null&&indexables.length>0){
      items.addItem(newDebugContentNode("indexables",indexables()));
      items.addItem(newDebugContentNode("index",indices[0]));
      items.addItem(newDebugContentNode("indexed",indexed()));
      items.addItem(newDebugContentNode("indexTitles",
      		coupler.newIndexableTitles(this)));
    }
    return items.items();
  }
	/**
	Creates an indexing with a default coupler. 
	@param title passed to core constructor
	@param indexables  passed to core constructor
	@param index passed to {@link #setIndex(int)}
	 */
	public static SIndexing newDefault(String title,Object[]indexables,int index){
		return new SIndexing(title,indexables,index,new Coupler());
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
