package facets.core.superficial.app;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargetCore;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.datatree.TypedNode;
/**
{@link STarget} with indexing of {@link STarget} children. 
<p>{@link IndexingTarget} defines key behaviour needed by 
{@link facets.core.superficial.app.SAreaTarget}, based on an 
{@link SIndexing} of {@link facets.core.superficial.STarget} children. 
 */
public abstract class IndexingTarget extends TargetCore{
	private SIndexing indexing;
  private IndexingTarget parent;
  /**
	Unique constructor. 
	@param title passed to the superclass 
	@param indexing passed to {@link #setIndexing(SIndexing)}
	 */
	protected IndexingTarget(String title,SIndexing indexing){
		super(title!=null?title:indexing!=null?indexing.title():null);
		setIndexing(indexing);
	}
	/**
	Sets the {@link SIndexing} containing the {@link STarget} children of 
	the {@link IndexingTarget}. 
	@param indexing will be returned by {@link #indexing()}; its indexables
	will be returned by {@link #indexableTargets()} 
	and its {@link SIndexing#indexed()} by {@link #indexedTarget()}; 
	the indexables must therefore be a {@link STarget}[]
	 */
	public void setIndexing(SIndexing indexing){
		if(indexing==null)throw new IllegalArgumentException("Null children in "+Debug.info(this));
		if(!(indexing.indexables()instanceof STarget[]))
		  throw new IllegalArgumentException("Bad children "+Debug.info(indexing.indexables()));
		(this.indexing=indexing).setMonitor(this);
	  STarget[]targets=indexableTargets();
	  for(int i=0;i<targets.length;i++){
	  	if(targets[i]==this)throw new IllegalArgumentException("Bad child in "+Debug.info(this));
	    if(targets[i]instanceof IndexingTarget)
				((IndexingTarget)targets[i]).parent=this;
		}
	}
	/**
	The indexing last set with {@link #setIndexing(SIndexing)}. 
	 */
	final public SIndexing indexing(){
	  if(indexing==null)throw new IllegalStateException("Null indexing in "+Debug.info(this));
	  return indexing;
	}
  /**
  Convenience cast for the <code>indexables</code> of the indexing. 
  @return <code>indexing().indexables()</code> cast to a 
  <code>{@link STarget}[]</code>
   */
  final public STarget[]indexableTargets(){
		return(STarget[])indexing().indexables();
	}
  /**
  Convenience cast for the <code>indexed</code> of the indexing. 
  @return <code>indexing().indexed()</code> cast to a 
  {@link STarget}
   */
	final public STarget indexedTarget(){
		return(STarget)indexing().indexed();
	}
  /**
	Overrides superclass method. 
	<p>Must return an {@link IndexingTargeter}. 
	 */
	public abstract STargeter newTargeter();
	/**
	Any containing {@link IndexingTarget}. 
	@return the parent which may be <code>null</code>
	 */
	final public IndexingTarget parent(){
		return parent;
	}
	/**
  Overrides superclass method. 
  <p>Ensures that only the indexed child is live (but ignores viewers).  
   */
  final public void setLive(boolean live){
		super.setLive(live);
		STarget indexed=indexedTarget();
		for(STarget child:indexableTargets())
			if(!(child instanceof SViewer))child.setLive(child==indexed?isLive():false);
			else if(false)traceDebug(".setLive: child=",child);
	}
	public String toString(){
    STarget child=indexedTarget();
    return Debug.info(this)+(false&&child instanceof IndexingTarget?" \n\t<"+child+">":"");
  }
	protected final boolean notifiesTargeter(){
		return true;
	}
	protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    STarget[]indexables=indexableTargets();
    if(indexables.length>0) {
      items.addItem(newDebugContentNode("indexableTargets",indexables));
      items.addItem(newDebugContentNode("indexedTarget",indexedTarget()));
    }
    if(indexing!=null)
      items.addItem(newDebugContentNode("indexing",indexing));
    return items.items();
  }
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

