package facets.core.superficial.app;
import static facets.util.Debug.*;
import static facets.util.app.Events.*;
import facets.core.superficial.Facetable;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.Notifying;
import facets.core.superficial.SFacet;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargetCore;
import facets.core.superficial.Notifying.Impact;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Util;
import facets.util.app.Events;
import facets.util.datatree.TypedNode;
/**
Represents viewers and other surface areas to the targeter tree. 
<p>{@link SAreaTarget} manages updating and focus for trees of viewers 
	and other facets, being the target of a directly attached facet 
	which it represents within an <a href="#AreaTrees">area target tree</a>. 
<p>Much key functionality is defined in the {@link IndexingTarget} superclass. </p>
<h3><a name="AreaTrees"></a>Area target trees</h3>
<p>The {@link IndexingTarget#indexableTargets()} of an {@link SAreaTarget} must 
	comprise one of</p>
<ul>
	<li>a single {@link SFrameTarget} or {@link ViewerTarget} 
	<li> one or more child {@link SAreaTarget}s 
</ul>
<p>A tree of {@link SAreaTarget}s therefore terminates in {@link SFrameTarget} 
	or {@link ViewerTarget} leaves. </p>
<p>The facets attached to and represented in the application by 
	{@link FacetedTarget} ({@link SAreaTarget} 
	and {@link ViewerTarget}) members of a tree generally form a containment hierarchy 
	of tabs/windows and panes. 
 */
public class SAreaTarget extends IndexingTarget implements FacetedTarget{
  public static final int AREA_LOWEST=Integer.MAX_VALUE;
	public static final STarget[]EMPTY_AREA_CHILDREN
		={new SFrameTarget("Empty area")};
  private SFacet facet;
	/**
  Core constructor which sets indexing. 
   @param title passed to superclass
    @param children passed to superclass
   */
  public SAreaTarget(String title,SIndexing children){
    super(title,children);
  }
	/**
  Convenience constructor setting indexable children. 
  @param title passed to the core constructor 
  @param indexableChildren  passed to the core constructor in an 
  {@link SIndexing} with a default coupler. 
   */
  public SAreaTarget(String title,STarget...indexableChildren){
  	this(title,SIndexing.newDefault(title,indexableChildren,0));
	}
	/**
	Overrides superclass method. 
	@param children must return 
	as its <code>indexables</code> a non-empty {@link STarget}[] 
	which is either 
	<ul>
		<li>single-member {@link STarget}[] containing either 
		a {@link ViewerTarget} or a {@link SFrameTarget}</li>
		<li>an <code>AreaTarget[]</code> </li>
	</ul>
	 */
	final public void setIndexing(SIndexing children){
		super.setIndexing(children);
		STarget[]targets=indexableTargets();
		if(targets.length>1){
			if(!(targets instanceof SAreaTarget[]))
				throw new IllegalArgumentException("Multiple non-area children in "+info(this));
		}
		else{
			STarget singleChild=targets[0];
			if(singleChild instanceof ViewerTarget)
				((ViewerTarget)singleChild).setAreaParent(this);
			else if(!(singleChild instanceof SFrameTarget
					||singleChild instanceof SAreaTarget))
				throw new IllegalArgumentException("Bad single child in "+info(this));
		}
	}
	/**
	Must return an {@link AreaTargeter}. 
	 */
	public STargeter newTargeter(){
		return new AreaTargeter(this);
	}
	final public void attachFacet(SFacet facet){
	  if(this.facet!=null&&this.facet!=facet)throw new RuntimeException(
	      "Facet already set in "+info(this)+", can't replace \n"+
	      info(this.facet)+" with "+info(facet)
	    );
	  else this.facet=facet;
	  if(false)trace("\nSAreaTarget: attached ",info(facet)+" to\n\t"+info(this));
	}
	final public SFacet attachedFacet(){
		if(facet==null)throw new IllegalStateException("Null facet in "+info(this)+" "+title()
				+(false?"":"\nparent="+parent()+"\ndescendants="+info(descendants())));
		return facet;
	}
	/**
	Implements interface method. 
	<p>In addition to to retargeting any {@link #attachedFacet()}, calls
	{@link Facetable#retargetFacets(Notifying.Impact)} in all {@link #indexableTargets()},
	starting with {@link #indexedTarget()} to minimise apparent latency. 
	 */
	final public void retargetFacets(Impact impact){
	  STarget children[]=indexableTargets(),indexed=indexedTarget();
	  if(indexed instanceof Facetable)((Facetable)indexed).retargetFacets(impact);
	  for(STarget child:children)
	  	if(child!=indexed)((SAreaTarget)child).retargetFacets(impact);
	  if(facet!=null){
	  	facet.retarget(this,impact);
	  	if(trace)traceEvent("Retargeted facet in "+this);
	  }
	}
	final public boolean isActive(){
	  boolean inPath=true;
	  IndexingTarget parent=this.parent(),child=this;
	  while(inPath&&parent!=null){
	    inPath&=child==parent.indexing().indexed();
	    if(false)trace(".isActive: child="+child+" inPath="+inPath);
	    child=parent;
	    parent=parent.parent();
	  }
	  return inPath;
	}
	final public void ensureActive(boolean notify){
		IndexingTarget parent=this.parent();
		if(parent==null||isActive())return;
		((SAreaTarget)parent).ensureActive(false);
		SIndexing indexing=parent.indexing();
		if(this!=indexing.indexed()){
			indexing.setIndexed(this);
			if(notify)notifyMonitor(Impact.ACTIVE);
		}
	}
	/**
	Convenience method. 
	@return {@link facets.core.superficial.STarget.Targeted#targets()} in 
	{@link #attachedFacet()}
	 */
	public final STarget[]facetTargets(){
		return((STarget.Targeted)attachedFacet()).targets();
	}
	/**
	The {@link FacetedTarget} representing the contained viewer or area facet 
	that should have the focus. 
	<p>Checks the {@link #indexedTarget()} of this {@link SAreaTarget} 
	and its descendents until it finds the last that is a {@link FacetedTarget}.
	 */
	final public FacetedTarget activeFacet(){
		FacetedTarget facet=this;
		STarget child;
		while(facet instanceof SAreaTarget){
			child=((SAreaTarget)facet).indexedTarget();
			if(!(child instanceof FacetedTarget))break;
			facet=(FacetedTarget)child;
		}
	  return facet;
	}
	/**
	The {@link SFrameTarget} framing the content exposed by 
	the active facet. 
	<p>Checks the {@link #indexedTarget()} of this {@link SAreaTarget} 
	and its descendents until it finds one that is not an {@link SAreaTarget};
	returns this if not a {@link ViewerTarget}, otherwise its
	{@link ViewerTarget#viewable}.
	 */
	final public SFrameTarget activeContent(){
		STarget check=this;
	  while(check instanceof SAreaTarget)
	  	check=((SAreaTarget)check).indexedTarget();
	  return (SFrameTarget)
	  	(check instanceof ViewerTarget?((ViewerTarget)check).viewable:check);
	}
	/**
	Depth-first traversal of this area tree, including the root. 
	 */
	public final STarget[]descendants(){
		ItemList<STarget>descendents=new ItemList(STarget.class);
		descendents.addItem(this);
		STarget[]children=indexableTargets();
		for(int i=0;i<children.length;i++)
			if(children[i] instanceof SAreaTarget)
				descendents.addItems(((SAreaTarget)children[i]).descendants());
			else descendents.addItem(children[i]);
		return descendents.items();
	}
	/**
	Attaches existing facet to a new area tree. 
	<p>Also applies the old area path to the new tree. 
	@param rootThen heads the area tree to which the existing facet are attached
	 */
	public final void attachThenFacets(SAreaTarget rootThen){
	  STarget[]descentThen=rootThen.descendants(),
			descentNow=descendants();
		if(false)trace(".attachThenFacets:\n",
					arrayInfo(descentThen)+"\n"+arrayInfo(descentNow));
		for(int i=0;i<descentNow.length;i++)
			if(descentNow[i]instanceof FacetedTarget){
				FacetedTarget facetedNow=(FacetedTarget)descentNow[i],
					facetedThen=(FacetedTarget)descentThen[i];
				facetedNow.attachFacet(facetedThen.attachedFacet());
				if(facetedNow instanceof SViewer)facetedNow.setLive(facetedThen.isLive());
				if(false)traceDebug(".attachThenFacets: ",descentNow[i]);
			}
		Object pathTarget=new AreaPath(rootThen).target(this);
		if(false)trace(".attachThenFacets: ",
				info(rootThen.activeFacet())+", "+info(pathTarget));
		((FacetedTarget)pathTarget).ensureActive(false);
	}
	/**
	Convenience method to a wrap single viewer in area. 
	<p>Passes to core constructor the <code>title</code> of <code>viewerFrame</code>,
	which is wrapped in an {@link SIndexing} of the same title. 
	 */
	final public static SAreaTarget newViewerArea(ViewerTarget viewerTarget){
	  String title=viewerTarget.title();
	  return new SAreaTarget(title,new ViewerTarget[]{viewerTarget});
	}
	protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    if(facet!=null)
      items.addItem(newDebugContentNode("area",facet));
    return items.items();
  }
	final public SAreaTarget areaParent(){
		return (SAreaTarget)parent();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

