package facets.core.superficial.app;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifying;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargetCore;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.TargeterCore.TargetType;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.StringFlags;
import facets.util.Util;
import facets.util.datatree.TypedNode;
/**
Heads the targeter tree for a content area. 
<p>The {@link SAreaTarget} root of each content area tree in a surface 
	must return from {@link SAreaTarget#newTargeter()} an appropriate 
	{@link SContentRootTargeter} to create and manage the targeter tree for all content 
	areas of that type. 
<p>Content areas for applications will generally, and those for dialogs 
	frequently, include one or more viewers exposing a {@link ViewableFrame}. 
<p>In addition to creating and retargeting its own <code>elements</code> on 
	those of its {@link SAreaTarget} target, {@link SContentRootTargeter} 
	applies specialised children on target trees representing (where present) 
<ul>
	<li>the content exposed by the area tree
	<li>the current selection within this content
	<li>the currently active viewer for the content
	<li>both the currently active view and all views used by the viewer
</ul>
<p>These child targeter trees provide attachment points for simple facets 
	forming panels, menus etc exposing their targets; viewer and area facets 
	for the surface should already be attached to the area target tree.
 */
public class SContentRootTargeter extends AreaTargeter{
  private STargeter viewer,content,view,views,selection,stateLinks[];
  public SContentRootTargeter(TargetType type){
		super(type);
	}
  public void retarget(STarget target,Impact impact){
  	super.retarget(target,impact);  	
  	SAreaTarget root=(SAreaTarget)target;
    SFrameTarget contentFrame=root.activeContent();
		content=retargetedTargeter(contentFrame,impact);
		if(contentFrame instanceof SelectingFrame)
			selection=retargetedTargeter(((SelectingFrame)contentFrame).selectionFrame(),
					impact);			
		FacetedTarget faceted=root.activeFacet();
  	if(!(faceted instanceof ViewerTarget))return;
  	ViewerTarget viewerTarget=(ViewerTarget)faceted;
    viewer=retargetedTargeter(viewerTarget,impact);
    STarget viewTarget=viewerTarget.viewFrame();
    view=retargetedTargeter(viewTarget,impact);
  	STarget viewtarget=viewerTarget.views;
  	if(viewtarget!=viewTarget){
      if(views==null)views=((TargetCore)viewtarget).newTargeter();
      views.retarget(viewtarget,impact);
  	}
  }
  /**
  Extends superclass behaviour by calling {@link #retargetFacets(Notifying.Impact)} in 
  its specialised targeter members.  
   */
  public void retargetFacets(Impact impact){
    super.retargetFacets(impact);
	  if(content==null)throw new IllegalStateException("No content in "+Debug.info(this));
    if(false)content.retargetFacets(impact);
    if(false&&selection!=null)selection.retargetFacets(impact);
    if(viewer==null)return;
    if(viewer.target()!=((SAreaTarget)target()).activeFacet())viewer.retargetFacets(impact);
    view.retargetFacets(impact);
    if(views!=null)views.retargetFacets(impact);
  }
  /**
	{@link STargeter} for the content exposed by the target's tree. 
	<p>The target of <code>content</code> is the {@link SFrameTarget} 
	returned as {@link SAreaTarget#activeContent()} 
	by the target of this {@link SContentRootTargeter}.
	 */
	final public STargeter content(){
	  if(content==null)throw new IllegalStateException("No content in "+Debug.info(this));
	  return content;
	}
	/**
	{@link STargeter} for the currently active view. 
	<p>The target of <code>view</code> is the {@link facets.core.superficial.SFrameTarget} 
    returned as <code>activeView</code> by the target of <code>viewer</code>.
   */
  final public STargeter view(){
    if(view==null)throw new IllegalStateException("No view in "+Debug.info(this));
    return view;
  }
  /**
	{@link STargeter} for the currently active viewer. 
	<p>The target of <code>viewer</code> is the {@link facets.core.superficial.app.ViewerTarget} 
    returned as <code>activeViewer</code> by the target of the 
    {@link SContentRootTargeter}.
   */
  final public STargeter viewer(){
    if(viewer==null)throw new IllegalStateException("No viewer in "+Debug.info(this));
    return viewer;
  }
  /**
	{@link STargeter} for the currently active views target. 
	<p>The target of <code>views</code> is the {@link facets.core.superficial.STarget} 
	    returned as <code>views</code> by the target of <code>viewer</code>.
	   */
  final public STargeter views(){
    return views!=null?views:view();
  }
	/**
	{@link STargeter} for the current selection. 
	<p>The target of <code>selection</code> is the {@link SFrameTarget} 
	    returned as {@link facets.core.superficial.app.SelectingFrame#selectionFrame()} by 
	    the target of {@link #content()}.
	   */
  final public STargeter selection(){
	  if(selection==null)throw new IllegalStateException("No selection in "+Debug.info(this));
	  return selection;
	}
	protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    if(viewer!=null)
      items.addItem(newDebugContentNode("viewer",viewer));
    if(views!=null)
      items.addItem(newDebugContentNode("views",views));
    if(view!=null)
      items.addItem(newDebugContentNode("view",view));
    if(content!=null)
      items.addItem(newDebugContentNode("content",content));
    if(selection!=null)
      items.addItem(newDebugContentNode("selection",selection));
    return items.items();
  }
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

