package facets.facet.app;
import facets.core.app.AppActions;
import facets.core.app.AppHost;
import facets.core.app.AppHost.LayoutFeatures;
import facets.core.app.AppSpecifier;
import facets.core.app.AppSurface;
import facets.core.app.AppSurface.ContentStyle;
import facets.core.app.SurfaceServices;
import facets.core.app.ViewerContenter;
import facets.core.superficial.SFacet;
import facets.core.superficial.STargeter;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SHost;
import facets.facet.FacetFactory;
import facets.facet.FacetFactory.ToolkitType;
import facets.util.Debug;
import facets.util.app.AppValues;
	/**
	{@link AppSpecifier} that builds and launches the {@link FacetAppSurface} that it specifies. 
	<p>A {@link FacetAppSpecifier} simplifies definition of a simple desktop application 
	defined by itself and by a {@link FacetAppValues}, {@link FacetAppSurface},  
	{@link ViewerContenter} and {@link SHost.FacetLayout}
</ul>
<p>These components are assembled into a complete GUI surface as follows:
<ol>
<li>A {@link FacetAppValues} is constructed in <code>main</code>, typically
redefining some of its default values.
<li>The {@link FacetAppValues} is passed to the constructor of the concrete 
{@link FacetAppSpecifier} and {@link #buildAndLaunchApp(String[])}
is called. 
<li>The {@link FacetAppSpecifier} obtains from {@link #newApp(FacetFactory)}
a concrete {@link FacetAppSurface} which will typically configure itself  
from both the {@link FacetAppSpecifier} and its {@link AppSpecifier#values}. 
<li>The {@link FacetAppSurface} is launched via {@link WindowAppHost#runHostedApp()}
in the return of {@link FacetAppSurface#host()}. 
<li>Either before or after launch, and as required based on user input,
 the {@link FacetAppSurface} creates {@link ViewerContenter}s in 
 {@link AppSurface#newContenter(Object)}, which in turn create the application content
 and define the user interface.
<li>The {@link SHost.FacetLayout} defining content-specific surface features
is returned by the first {@link ViewerContenter} for the content type
from {@link ViewerContenter#newContentLayout(SHost,SFacet, SContentRootTargeter)}. 
	</ol>
		*/
	public abstract class FacetAppSpecifier extends AppSpecifier{
		/** Debug flag. */
		public static final String ARG_NO_FILES="noFiles",ARG_NO_CREATE="noCreate";
		public FacetAppSpecifier(FacetAppValues values){
			super(values);
		}
		/**
		Calls {@link #newApp(FacetFactory)} and then {@link AppHost#runHostedApp()}. 
		@param args passed to {@link AppValues#readValues(String...)} 
		before creating app
		 */
		final public void buildAndLaunchApp(String[]args){
			values.readValues(args);
			((AppHost)newApp(newFacetFactory()).host()).runHostedApp();
		}
		/**
		Final implementation for desktop applications. 
		@return {@link FacetFactory#newDesktopCore(FacetAppValues, ToolkitType)}
		 */
		final protected FacetFactory newFacetFactory(){
			return FacetFactory.newDesktopCore((FacetAppValues)values,toolkitType());
		}
		/**
		Called from {@link #buildAndLaunchApp(String...)}.
		@param ff created in {@link #newFacetFactory()}
		@return a {@link FacetAppSurface} that will create content and the surface
		to expose it
		 */
		protected abstract FacetAppSurface newApp(FacetFactory ff);
		/**
		Defines the GUI widgets to be used for the application surface. 
		@return a {@link ToolkitType} specifying the toolkit to be used by {@link FacetFactory};
		by default {@link ToolkitType#SWING_SYSTEM}
		 */
		public ToolkitType toolkitType(){
			return ToolkitType.SWING_SYSTEM;
		}
		/**
		Defines default returns of {@link #newActions()} and {@link #contentStyle()}. 
		@return by default <code>true</code> unless contradicted using {@link #ARG_NO_FILES}  
		 */
		public boolean isFileApp(){
			return !values.nature(true).getBoolean(ARG_NO_FILES);
		}
		/**
		Re-implementation setting positive default. 
		@return by default <code>true</code> unless contradicted using {@link #ARG_NO_CREATE}  
		 */
		@Override
		public boolean canCreateContent(){
			return !values.nature(true).getBoolean(ARG_NO_CREATE);
		}
		/**
		Implements abstract method. 
		@return by default either a {@link FacetAppActions} or a {@link FileAppActions}
		depending on {@link #isFileApp()}
		 */
		@Override
		public AppActions newActions(){
			return isFileApp()?new FileAppActions():new FacetAppActions();
		}
		/**
		Re-implementation making instance checks. 
		@return by default <code>true</code> where {@link #canEditContent()} and {@link #isFileApp()} 
		both return <code>true</code>
		 */
		@Override
		public boolean canSaveContent(){
			return canEditContent()&&isFileApp();
		}
		/**
		Re-implementation looking at {@link #isFileApp()}. 
		@return by default {@link ContentStyle#DESKTOP}
		 */
		public ContentStyle contentStyle(){
			return !isFileApp()?ContentStyle.SINGLE:ContentStyle.DESKTOP;
		}
	}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
