package facets.facet.toolkit.swing;
import static facets.facet.FacetFactory.*;

import facets.core.superficial.SNumeric;
import facets.core.superficial.STarget;
import facets.core.superficial.STextual;
import facets.facet.FacetFactory;
import facets.facet.SwingPanelFacet;
import facets.facet.toolkit.Decoration;
import facets.facet.toolkit.KField;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.StringFlags;
import facets.util.Util;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.Timer;
import javax.swing.border.Border;
/**
{@link FieldText} encapsulates some complex logic to enable intuitive
user interaction: 
<ul>
	<li>appropriate responses to input and to change in focus 
	<li>(optional) interim notification as field text is edited 
</ul>
<table width="100%" border="1">
	<tr> 
		<td width="20%"><b>Event</b></td>
		<td width="40%"><b>Interim off</b></td>
		<td width="40%"><b>Interim on</b></td>
	</tr>
	<tr> 
		<td><i>Focus gained</i></td>
		<td colspan="2" align="center"> Ready field</td>
	</tr>
	<tr> 
		<td><i>Edit</i></td>
		<td> None</td>
		<td> (Create notifier,) revert invalid field, notify interim</td>
	</tr>
	<tr> 
		<td><i>Esc</i></td>
		<td> Revert field</td>
		<td> Revert field; notify interim</td>
	</tr>
	<tr> 
		<td><i>Enter, focus lost</i></td>
		<td colspan="2"> Revert invalid field, notify final, ready field</td>
	</tr>
</table>
<p>It also supports undo/redo(Ctrl-Z/Y).
 */
class FieldText extends Widget implements KField{
	final JFormattedTextField field=new JFormattedTextField();
	private final JLabel label;
	private final FocusRequester focusRequestor=new FocusRequester(field);
	private final boolean formUsage,updateInterim;
  protected String lastSetText="",lastEditText="";
	private boolean indeterminate;
  private Timer notifier;
	FieldText(final KitFacet facet,int cols,Kit kit,StringFlags hints){
		super(facet,null);
  	formUsage=hints.includeFlag(HINT_USAGE_FORM);
		STextual textual=findTextual();
  	updateInterim=textual!=null&&textual.coupler.updateInterim(textual);
  	boolean bare=hints.includeFlag(FacetFactory.HINT_BARE);
  	JPanel swing=new JPanel(new FlowLayout(FlowLayout.LEADING,0,0));
		KitSwing.adjustFont(swing);
		decorations=kit.decoration(facet.title(),hints);
		if(bare)label=null;
  	else{
			label=new JLabel(decorations.caption);
			label.setDisplayedMnemonic(decorations.mnemonic);
  		KitSwing.adjustFont(label);
  		swingMap.put(SwingPanelFacet.KEY_LABEL,label);
  		swing.add(Box.createHorizontalStrut(3));
  		swing.add(label);
  		swing.add(Box.createHorizontalStrut(3));
  		Kit.widgets++;
  	}
  	swingMap.put(SwingPanelFacet.KEY_LABELLED,field);
  	swing.add(field);
  	Kit.widgets++;
  	setSwing(swing);
		KitSwing.adjustFont(field);
		field.setColumns(cols);
		field.setEditable(false);
		field.addFocusListener(new FocusListener(){
			public void focusGained(FocusEvent e){
				if(true)field.selectAll();
				else if(false)readyField(lastSetText=text());
			}
			public void focusLost(FocusEvent e){notifyFinalValid();}
		});
		field.addKeyListener(new KeyAdapter(){
      public void keyPressed(KeyEvent e){
      	boolean interim=updateInterim&&(true||notifyInterim);
      	int keyCode=e.getKeyCode();
				switch(keyCode){
      	case KeyEvent.VK_ESCAPE:
      		readyField(lastSetText);
      		field.selectAll();
					if(interim)facet.targetNotify(this,true);
      		break;
      	case KeyEvent.VK_ENTER:
      		notifyFinalValid();
      		break;
      	default:
	    		handleFieldInput(keyCode,interim);
      	}
      }
      public void keyTyped(KeyEvent e){
    		boolean ctrl=(e.getModifiers()&KeyEvent.CTRL_MASK)!=0;
    		if(false)trace(".keyPressed: ",e.getKeyChar());
      }
    });
	}
  public void setEnabled(boolean enabled){
  	field.setFocusable(enabled);
		if(formUsage){
			field.setEditable(enabled);
			if(label!=null)label.setDisplayedMnemonic(enabled?
					decorations.mnemonic:0);
		}
		else{
			field.setEnabled(enabled);
			if(label!=null)label.setEnabled(enabled);
		}
	}
	private void handleFieldInput(int keyCode, boolean interim){
		lastEditText=field.getText();
		if(!interim)return;
		if(notifier==null)notifier=new Timer(dragNotifyPause,
				new ActionListener(){		
			public void actionPerformed(ActionEvent e) {
				validateFieldText();
				facet().targetNotify(this,true);
			}
		}){
			public boolean isRepeats(){return false;}
		};
		notifier.stop();
		notifier.start();
	}
	private void notifyFinalValid(){
		validateFieldText();
		if(notifier!=null)notifier.stop();
		notifier=null;
		if(!field.getText().equals(lastSetText))facet().targetNotify(this,false);
	}
	private void validateFieldText(){
		String text=field.getText();
		STextual textual=findTextual();
		if(textual!=null&&!textual.coupler.isValidText(textual,text))
			readyField(lastEditText);
	}
	private STextual findTextual() {
		KitFacet facet=facet();
		STarget target=facet.target();
		if(target instanceof STextual)return(STextual)target;
		if(target instanceof SNumeric)return null;
		target=target.elements()[0];
		return target instanceof STextual?(STextual)target:null;
	}
	public void setText(String text){
		if(text==null)throw new IllegalArgumentException(
				"Null text in "+Debug.info(this));
		if(notifier==null)readyField(lastSetText=text);
	}
	private void readyField(String text){
		field.setText(text);
		field.setCaretPosition(text.length());
	}
  public void setIndeterminate(boolean on){
		if(notifier!=null)return;
		readyField(lastSetText=(indeterminate=on)?"":lastSetText);
	}
	public String text(){
		return field.getText();
	}
	protected FieldText(KitFacet facet,int cols,
  		boolean alignRight,Kit toolkit,StringFlags hints){
		this(facet,cols,toolkit,hints);
		if(alignRight)field.setHorizontalAlignment(JFormattedTextField.RIGHT);
	}
  public void makeEditable(){field.setEditable(true);}
	public void requestFocus(){
		focusRequestor.startRequesting();
	}
	public String toString(){
		return super.toString()+" "+text();
	}
	public void setValue(double value){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public double value(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

