package facets.idiom;
import static facets.core.app.PathSelection.*;
import static facets.util.datatree.Nodes.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppSpecifier;
import facets.core.app.ListView;
import facets.core.app.NodeViewable;
import facets.core.app.PathSelection;
import facets.core.app.TreeView;
import facets.core.superficial.SFacet;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.SelectingFrame;
import facets.facet.FacetFactory;
import facets.facet.app.FacetAppSpecifier;
import facets.facet.app.FacetAppValues;
import facets.facet.app.tree.TreeAppSpecifier;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.OffsetPath;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.app.AppValues;
import facets.util.datatree.DataNode;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
import facets.util.datatree.XmlSpecifier;
/**
Utility application for viewing and editing simple tree content.
<p>{@link XmlView} demonstrates the range of functionality provided by  
{@link TreeAppSpecifier}:
<ul>
  <li>viewing and editing of XML files
  <li>search by XML tags, content, attributes and attribute values 
  <li>changing XML tag names case by case or globally 
  <li>save and read of compressed files 
  <li>encoding and decoding of content as ASCII text 
  <li>multiple document interface 
  <li>quick-expand tree display 
  <li>status bar with information on selected element
  <li>content selection restored on file reload 
</ul>
<p>Behaviour and appearance of applications based on {@link TreeAppSpecifier} 
can be modified by reimplementing methods as listed below; 
the code of {@link XmlView} itself provides examples which can be switched on by 
passing {@link #ARG_DEMO} to {@link #main(String[])}. 

<p>In {@link TreeAppSpecifier}:
<ul>
<li>{@link #xmlPolicy()} to redefine {@link XmlSpecifier}s used in file dialogs
<li>{@link #newContentViews(NodeViewable)} to add or customise content viewers
<li>{@link #newContentRootTargets()} to define additional {@link STarget}s
 to be exposed in the {@value facets.facet.app.tree.TreeTargets#TITLE_MENU} menu; 
 and {@link #newTreeMenuItems(FacetFactory, STargeter[], STargeter[])}
  to define the {@link SFacet}s exposing them.  
<li>{@link #getInternalContentSource()} to redefine default content
</ul>
<p>In {@link FacetAppSpecifier}:
<ul>
<li>{@link #canCreateContent()} to toggle content creation 
<li>{@link #isFileApp()} to toggle file open/close 
<li>{@link #canSaveContent()} to toggle file save/restore 
<li>{@link #contentStyle()} to redefine application style
<li>{@link #toolkitType()} to redefine look-and-feel
</ul>
<p>In {@link AppSpecifier}:
<ul>
<li>{@link #canEditContent()} to toggle editing
<li>{@link #canOverwriteContent()} to toggle overwriting files
<li>{@link #offersHelp()} if help resources have been created  
</ul>
<p>In {@link Tracer}:
<ul>
<li>{@link #traceOutput(String)} to toggle debug tracing
</ul>
 */
final public class XmlView extends TreeAppSpecifier{
	public static final String ARG_DEMO="demo";
	private final DataNode demoNode=new DataNode("Demo",values.appName);
	public XmlView(FacetAppValues values){
		super(values);
	}
	@Override
	public boolean isFileApp(){
		return noDemo()?super.isFileApp():false;
	}
	private boolean noDemo(){
		return !values.nature(false).getBoolean(ARG_DEMO);
	}
	@Override
	protected Object getInternalContentSource(){
		if(noDemo())return super.getInternalContentSource();
		demoNode.setChildren(values.nature(false),values.state());
		return demoNode;
	}
	@Override
	protected SView[]newContentViews(NodeViewable viewable){
		if(noDemo())return super.newContentViews(viewable);
		viewable.defineSelection(values.nature(false));
		final boolean liveViews=canEditContent();
		return new SView[]{
				new TreeView("Outline"){
					public boolean isLive(){
						return liveViews;
					};
				},
				new TreeView("Node"){
					@Override
					public SSelection newViewerSelection(SViewer viewer,
							SSelection viewable){
						PathSelection pathsIn=(PathSelection)viewable;
						Object[]members=pathMembers(pathsIn,0);
						return procrust(pathsIn,members[members.length-1]);
					}

					@Override
					public boolean isLive(){
						return liveViews;
					};
				}
		};
	}
	@Override
	protected STarget[]newContentRootTargets(){
		if(noDemo())return super.newContentRootTargets();
		return new STarget[]{
			new SIndexing("Select Node",demoNode.children(),
					0,new SIndexing.Coupler(){
				@Override
				public String[]newIndexableTitles(SIndexing i){
					String titles="";
					for(TypedNode node:(TypedNode[])i.indexables())
						titles+=node.type()+"\n";
					return titles.split("\n");
				}
				@Override
				public void indexSet(SIndexing i){
					DataNode node=(DataNode)i.indexed();
					((ActionAppSurface)app()).dialogs().infoMessage("Selecting Node",
							node.type());
					((SelectingFrame)app().findActiveContent().contentFrame()).defineSelection(node);
				}
			})
		};
	}
	@Override
	protected SFacet[]newTreeMenuItems(FacetFactory f,STargeter[]treeLinks,
			STargeter[]contentLinks){
		if(noDemo())return super.newTreeMenuItems(f,treeLinks,contentLinks);
		else return new SFacet[]{
			f.indexingRadioButtonMenu(contentLinks[0],FacetFactory.HINT_NONE)
		};
	}
	public static void main(String[]args)throws Exception{
		final boolean releaseReady=true;
		new XmlView(new FacetAppValues(XmlView.class){
			@Override
			protected void addNatureDefaults(ValueNode root){
				super.addNatureDefaults(root);
				mergeContents((ValueNode)child(root,PATH_CORE),new Object[]{
					NATURE_DEBUG+"="+!releaseReady,
				});
			}
		}).buildAndLaunchApp(args);
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
