package facets.util;
import java.io.Serializable;
/**
Has state that can be set, copied and checked for deep equality.  
<p>{@link Stateful} encapsulates management of an object whose state may change,
defining methods for 
<ul>
<li>changing state while maintaining identity
<li>deep copying and equality checking
<li>recording and checking for state changes   
</ul>
 */
public interface Stateful extends Serializable,Titled{
	  /**
	Set state based on <code>src</code>. 
	@param src must be non-null and interpretable by the implementation 
		as state information; it will usually be another instance of the
	{@link Stateful}'s class. 
	   */
	void setState(Object src);
		/**
	Create a copy of the {@link Stateful}. 
		<p>Return one of: </p>
		<ul>
			<li>a deep copy of the {@link Stateful} suitable for passing to <code>setState</code></li>
			<li>the {@link Stateful} itself to enable such a copy to be created</li>
			<li> <code>null</code> to signal that the {@link Stateful} has 
			in fact no settable state </li>
		</ul>
	 */
	Stateful copyState();
  /**
	Does the other {@link Stateful} have the same state?
	<p>Allows deep checking to be implemented where doing so in 
	{@link Object#equals(Object)} is too expensive.  
	@param s to compare
	 */
	boolean stateEquals(Stateful s);
	/**
	Create a new value for return by {@link #stateStamp()}.  
 @return the new value
 */
	Object updateStateStamp();
  /**
	Return a value guaranteeing that any instance with the same value has 
	the same state. 
	 */
	Object stateStamp();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

