package facets.util.app;
import static facets.util.Debug.*;
import static facets.util.Times.*;
import static facets.util.datatree.Nodes.*;
import static facets.util.datatree.ValueNode.*;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.TextLines;
import facets.util.Tracer;
import facets.util.datatree.DataConstants;
import facets.util.datatree.DataNode;
import facets.util.datatree.XmlPolicy;
import facets.util.datatree.ExceptionNode;
import facets.util.datatree.NodeList;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
import facets.util.datatree.XmlDocRoot;
import facets.util.datatree.Nodes.TreeRoot;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.DateFormat;
import java.util.Date;
/**
Can persist/depersist configuration and session state/user preferences. 
<p>{@link AppValues} wraps two {@link ValueNode}s which can be used to 
read, maintain and store values on behalf of a client application. 
 */
public class AppValues extends Tracer{
	/**
	Type of root node, used by #{@link #storageFile(boolean)}. 
	 */
	public static final String TYPE_STATE="state",TYPE_NATURE="nature";
	/**
	Type/path of values node. 
	 */
	public static final String PATH_CORE="facets",PATH_APP="app",PATH_DEBUG="debug",
		PATH_CORE_APP[]={PATH_CORE,PATH_APP},PATH_CORE_DEBUG[]={PATH_CORE,PATH_DEBUG},
		PATH_RUNS="runs";
	/**
 	Key for setting {@link Debug#natureDebug}. 
	<p>Set to <code>false</code> for release binaries; to <code>true</code> in binary by passing
	 as argument. 
	 */
	public static final String NATURE_DEBUG="debug";
	/** Key for flag specifying instance behaviour. */
	public static final String NATURE_WRITABLE="natureWritable",
		NATURE_RECORD_RUNS="recordRuns",
		NATURE_NO_SYSTEM_ACCESS="noSystemAccess",
		KEY_TIMEOUT="timeoutBlock",KEY_TIMEOUT_SYSTEM="timeoutSystem";
	private static final DateFormat DATE_FORMAT=DateFormat.getDateInstance(),
		TIME_FORMAT=DateFormat.getTimeInstance();
	/**
	Global state node for debug values. 
	<p>Created empty, added as child of core state.  
	 */
	public static final ValueNode stateDebug=new ValueNode(PATH_DEBUG,UNTITLED);
	/**
	Simple name of the class passed to the constructor; expected to be the application name. 
	<p>Used by {@link #storageFile(boolean)}. 
	 */
	public final String appName;
	private final ValueNode nature=new ValueNode(TYPE_NATURE,UNTITLED),
		state=new ValueNode(TYPE_STATE,UNTITLED);
	/**
	Unique constructor. 
	<p>Builds data roots ready for {@link #readValues(String[])}:
	@param appClass should generally be the client application class; its simple name is
	used for the {@link #appName} field
	 */
	public AppValues(Class appClass){
		appName=appClass.getSimpleName();
	}
	/**
	Can set default configuration values.
	<p>Called from {@link #readValues(String[])}; 
	values added are treated as defaults for {@link #nature(boolean)}.
	<p>Default implementation is empty.  
	 */
	protected void addNatureDefaults(ValueNode root){}
	/**
	Can set default session/preferences values. 
	<p>Called from {@link #readValues(String[])}; 
	values added are treated as defaults for {@link #state(String...)}.  
	<p>Default implementation is empty.  
	@param root of the {@link #state(String...)} tree
	 */
	protected void addStateDefaults(ValueNode root){}
	/**
	Reads values from (in order) defaults, parameters and storage. 
	<p>Returns without effect after the first call; calls to {@link #nature(boolean)}
	and {@link #state(String...)} will fail before this method is called.  
	<p>Node paths are created before reading for
	<ul>
	<li>{@link #nature}: {@link #PATH_CORE} with basic values
	<li>{@link #state}: {@link #PATH_CORE_DEBUG} and {@link #PATH_CORE_APP}. 
	</ul>
	@param parameters passed from <code>main</code> or constructed from
	applet parameters; any member not a <i>key=value</i> pair is 
	interpreted as <i>parameter=true</i>.
	 */
	final public void readValues(String...parameters){
		if(nature.contents().length>0)return;
		nature.setContents(new Object[]{
			new ValueNode(PATH_CORE,new Object[]{
				NATURE_WRITABLE+"="+false,
				NATURE_RECORD_RUNS+"="+false,
			}),
		});
		addNatureDefaults(nature);
		boolean hasSystemAccess=hasSystemAccess();
		TextLines.setDefaultEncoding(hasSystemAccess);
		if(!appName.equals(""))readSourceFile(getClass().getClassLoader().getResource(
				storageFile(true).getPath()),new XmlDocRoot(nature,XML_POLICY));
		for(String par:parameters){
			if(par==null)continue;
			if(par.indexOf(KEY_EQUALS)<0)par+=KEY_EQUALS+KEY_TRUE;
			int equalsAt=par.indexOf(KEY_EQUALS);
			String[]pair={par.substring(0,equalsAt).trim(),
					par.substring(equalsAt+1).trim()};
			String key=pair[0],value=pair[1];
			nature(true).put(key,value);
			nature(false).put(key,value);
		}
		state.setContents(new Object[]{new ValueNode(PATH_CORE,new Object[]{
				stateDebug,
				new ValueNode(PATH_APP,UNTITLED)
			})
		});
		addStateDefaults(state);
		readSourceFile(!hasSystemAccess?
				null:storageFile(false),new XmlDocRoot(state,XML_POLICY));
		adjustClassValues();
		trace("Nature and state values read ");
		updateRuns(true);
	}
	/**
	Can refresh class variables etc from state values. 
	<p>Called from {@link #readValues(String[])}; defined <code>public</code> to allow calls
	when values are changed.
	<p>Default implementation sets {@link Debug#natureDebug}. 
	 */
	public void adjustClassValues(){
		natureDebug=nature(true).getOrPutBoolean(NATURE_DEBUG,natureDebug);
	}
	private static final XmlPolicy XML_POLICY=new XmlPolicy(){
		protected boolean treeAsXmlRoot(){
			return true;
		}
		protected boolean handleReadExceptions(){
			return false;
		} 
		protected ValueNode getTitleAttributeNames(){
			return newTitleAttributeNames("",new String[]{"Date=day"});
		}
	};
	private void readSourceFile(Object source,TreeRoot dataRoot){
		ValueNode values=(ValueNode)dataRoot.tree;
		Object[]defaults=values.contents();
		try{
			dataRoot.readFromSource(source instanceof File?
					new TextLines((File)source):new TextLines((URL)source));
		}catch(Exception e){
			boolean alertExceptions=false;
			values.setContents(!alertExceptions?defaults
					:Objects.join(Object.class,defaults,new Object[]{new ExceptionNode(e)}));
		}
		ValueNode merge=new ValueNode(DataConstants.TYPE_DATA,"Merge",defaults);
		mergeContents(merge,values.contents());
		values.setContents(merge.contents());
	}
	/**
	Attempts to persist current values in XML format. 
<ul>
	<li>	If {@link #NATURE_RECORD_RUNS} is <code>true</code>, adds a child to the 
	{@link #PATH_RUNS} child of {@link #PATH_CORE}.
	<li>	Will only write out the contents 
	of {@link #nature(boolean)} if {@link #NATURE_WRITABLE} is <code>true</code>;
	</ul>
	 */
	final public void tryWrite(String msg){
		updateRuns(false);
		if(!hasSystemAccess())trace("No system access to write values");
		else try{
			File storage=storageFile(false);
			new XmlDocRoot(state,XML_POLICY).writeToSink(storage);
			String msgCore="State values written to "+storage.getAbsolutePath();
			if(!msg.trim().equals(""))System.out.println(msg+msgCore);
			else trace(msg);
			if(!nature(true).getBoolean(NATURE_WRITABLE))return;
			storage=storageFile(true);
			new XmlDocRoot(nature,XML_POLICY).writeToSink(storage);
			trace("Nature values written to "+storage.getAbsolutePath());
		}catch(IOException e){
			System.out.println(msg+"No state values written");
		}
	}
	protected void traceOutput(String msg){
		if(false)traceOutputWithClass(msg);
	}
	private void updateRuns(boolean start){
		if(!nature(true).getBoolean(NATURE_RECORD_RUNS))return;
		DataNode runs=state(PATH_CORE,PATH_RUNS);
		TypedNode previous[]=runs.children(),
			latest=previous.length==0?null:previous[previous.length-1];
		ValueNode run=start?new ValueNode(DataConstants.TYPE_DATE,
				DATE_FORMAT.format(new Date()))
			:(ValueNode)latest;
		String timeValue=TIME_FORMAT.format(new Date());
		if(run!=latest){
			appendChild(runs,run);
			run.put("start",timeValue);
		}
		else run.put("stop",timeValue);
	}
	/**
	Defines application configuration or 'nature'. 
	 <p>Initial contents are set by 
	 <ol>
	 <li>reading the values returned by {@link #addNatureDefaults(ValueNode)}</li>
	 <li>attempting to merge values from an XML <i>.nature</i> file 
	 named after the <code>title</code> passed to the 
	 constructor and accessible as a system resource. </li>
	 <li>Adding parameters (to both root and <code>core</code> nodes) 
	 as key-value pairs, verbatim if containing '=',
	 otherwise as '<i>parameter</i>=true'. </li>
	 </ol>
	 <p>Value updates are lost after each session 
	 (which is why parameters are added to these values rather than to the 
	 ostensibly more appropriate {@link #state(String...)}). 
	  @param core returns the framework-specific sub-node
	 */
	final public ValueNode nature(boolean core){
		if(nature.contents().length==0)
			throw new IllegalStateException("Values not read in "+Debug.info(this));
		else return!core?nature:(ValueNode)child(nature,PATH_CORE);
	}
	/**
	Stores application session state and user preferences. 
	 <p>Initial contents are set by
	 <ol>
	 <li>reading the values returned by {@link #addStateDefaults(ValueNode)}</li>
	 <li>attempting to merge values from an XML <i>.state</i> file 
	 named after the <code>title</code> passed to the 
	 constructor and located in the start directory. </li>
	 </ol>
	 <p>Value updates can be written to the <i>.state</i> file 
	 with {@link #tryWrite(String)}. 
	  @param typePath specifies a path within the state tree to a node which is
	  created if required; no parameter specifies the root 
	 */
	final public ValueNode state(String...typePath){
		if(nature.contents().length==0)
			throw new IllegalStateException("Values not read in "+Debug.info(this));
		return typePath.length==0?state:guaranteedDescendant(state,typePath);
	}
	/**
	Can these values (and by implication the application) access the file system,
	clipboard etc?  
	@return value of {@link #NATURE_NO_SYSTEM_ACCESS} which can be passed as an
	argument 
	 */
	public boolean hasSystemAccess(){
		return !nature(true).getBoolean(NATURE_NO_SYSTEM_ACCESS);
	}
	/**
	File storing nature or state content. 
	<p>The file path is used when reading as a resource.  
	 */
	private File storageFile(boolean forNature){
		String type=forNature?TYPE_NATURE:TYPE_STATE;
		return new File(appName+"."+type+".xml");
	}
	/** 
	Reads value pairs from the console and writes them to XML. 
	<p>Output files are <i>MyAppValues.[nature|state].xml</i>; 
	implementations of {@link #addNatureDefaults(ValueNode)} and
	{@link #addStateDefaults(ValueNode)} demonstrate techniques for 
	constructing and modifying {@link ValueNode}s. 
	@param args read by {@link #readValues(String[])}
	 */
	static void main(String[]args)throws Exception{
		AppValues values=new AppValues(AppValues.class){
			protected void traceOutput(String msg){
				traceOutputWithClass(": "+msg);
			}
			protected void addStateDefaults(ValueNode root){
				super.addStateDefaults(root);
				root.put("aFlag",true);
				root.put("intArray",new int[]{1,2,3,4});
				if(false)new NodeList(root,true).add((TypedNode)nature(false).copyState());
			}
		};
		values.readValues(args);
		values.tryWrite("");
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

